#include "Test.h"
#include "Matrix.h"
#include <cmath>



bool testAssOp()
{
   Matrix<double> m1(2, 3);
   m1.Setze(1, 2, 14);

   Matrix<double> m2(12, 3);
   m2.Setze(3, 2, 21);

   m2 = m1;
   if (14 != m2.Lese(1, 2))
   {
      return false;
   }

   Matrix<double> m3(2, 5);
   m3.Setze(1, 4, -36);
   m3 = m1 = m2;
   if (14 != m2.Lese(1, 2) ||
       14 != m1.Lese(1, 2) ||
       14 != m3.Lese(1, 2))
   {
      return false;
   }

   // Eigenzuweisung testen
   Matrix<double> m4(3, 5);
   m4.Setze(2, 4, -36);
   m4 = m4;
   if (-36 != m4.Lese(2, 4))
   {
      return false;
   }

   return true;
}


/**
Es wird eine 3*4 Matrix angelegt und einige Werte
werden gesetzt, die dann ausgelesen werden.
Die Matrix wird dann in eine 8*2 Matrix gendert
und wiederum mit Werten belegt und ausgelesen werden.
Liefert das Auslesen die erwarteten Werte, wird true
geliefert
*/
bool test1()
{
   const int ze = 3;
   const int sp = 4;
   Matrix<double> m1(ze, sp);

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         m1.Setze(i, j, i*sp + j * 2);
      }
   } // for i

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         if (m1.Lese(i, j) != (i*sp + j * 2)) {
            return false;
         }
      }
   } // for i

   return true;
}


/**
siehe test1, aber nun mit int
*/
bool test1Int()
{
   const int ze = 3;
   const int sp = 4;
   Matrix<int> m1(ze, sp);

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         m1.Setze(i, j, i*sp + j * 2);
      }
   } // for i

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         if (m1.Lese(i, j) != (i*sp + j * 2)) {
            return false;
         }
      }
   } // for i

   return true;
}


/**
Es wird eine 3*4 Matrix angelegt und einige Werte
werden gesetzt, die dann ausgelesen werden.
Die Matrix wird dann in eine 8*2 Matrix gendert
und wiederum mit Werten belegt und ausgelesen werden.

Anschlieend wird sie in eine 8*2-Matrix gendert.
Liefert das Auslesen die erwarteten Werte, wird true
geliefert

*/
bool test2()
{
   int ze = 3;
   int sp = 4;
   Matrix<double> m1(ze, sp);

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         m1.Setze(i, j, i*sp + j * 2);
      }
   } // for i

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         if (m1.Lese(i, j) != (i*sp + j * 2)) {
            return false;
         }
      }
   } // for i

   ze = 8;
   sp = 2;
   m1.Resize(ze, sp);

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         m1.Setze(i, j, i*sp + j * 2);
      }
   } // for i

   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         if (m1.Lese(i, j) != (i*sp + j * 2)) {
            return false;
         }
      }
   } // for i


   return true;
}



/**
Es werden zwei Matrizen angelegt und die Summe mit Add
ermittelt. Es wird berprft, ob die Summe geliefert wird.
*/
bool testAdd()
{
   int ze = 3;
   int sp = 4;
   Matrix<double> m1(ze, sp);
   Matrix<double> m2(ze, sp);

   double wert1(2.4);
   double wert2(2.5);
   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         m1.Setze(i, j, wert1);
         m2.Setze(i, j, wert2);
      }
   } // for i

   Matrix<double> m3(ze, sp);
   double  EPS = 0.00001;
   Add(m1, m2, m3);
   for (int i = 0; i < ze; ++i) {
      for (int j = 0; j < sp; ++j) {
         if (fabs(m3.Lese(i, j) - (wert1 + wert2)) > EPS) {
            return false;
         }
      }
   } // for i

   return true;
}

